@@ -14,6 +14,7 @@ import json
 
 import subprocess
 import signal
+import threading
 
 import trace
 from ethoscope.hardware.input.cameras import OurPiCameraAsync, MovieVirtualCamera, V4L2Camera
@@ -497,11 +498,30 @@ class ControlThread(Thread):
                         hardware_connection, StimulatorClass, stimulator_kwargs, time_offset=0):
 
         #Here the stimulator passes args. Hardware connection was previously open as thread.
-        stimulators = [StimulatorClass(hardware_connection, **stimulator_kwargs) for _ in rois]
+        logging.info(f"Creating stimulators: StimulatorClass={StimulatorClass}, hardware_connection={hardware_connection}, stimulator_kwargs={stimulator_kwargs}")
+        logging.info(f"Number of ROIs: {len(rois)}")
+        
+        stimulators = []
+        for i, roi in enumerate(rois):
+            try:
+                stimulator = StimulatorClass(hardware_connection, **stimulator_kwargs)
+                logging.info(f"Successfully created stimulator {i+1}/{len(rois)}: {type(stimulator).__name__}")
+                stimulators.append(stimulator)
+            except Exception as e:
+                logging.error(f"Failed to create stimulator {i+1}/{len(rois)}: {e}")
+                # Fall back to DefaultStimulator
+                from ethoscope.stimulators.stimulators import DefaultStimulator
+                fallback_stimulator = DefaultStimulator(None)
+                logging.warning(f"Using DefaultStimulator fallback for ROI {i+1}")
+                stimulators.append(fallback_stimulator)
         
         kwargs = self._monit_kwargs.copy()
         kwargs.update(tracker_kwargs)
 
+        logging.info(f"Creating Monitor with {len(stimulators)} stimulators")
+        for i, stimulator in enumerate(stimulators):
+            logging.info(f"Stimulator {i+1}: {type(stimulator).__name__}")
+            
         self._monit = Monitor(camera, TrackerClass, rois,
                               reference_points = reference_points,
                               stimulators=stimulators,
@@ -562,8 +582,6 @@ class ControlThread(Thread):
         """
         CameraClass = self._option_dict["camera"]["class"]
         camera_kwargs = self._option_dict["camera"]["kwargs"]
-        ROIBuilderClass = self._option_dict["roi_builder"]["class"]
-        roi_builder_kwargs = self._option_dict["roi_builder"]["kwargs"]
 
         StimulatorClass = self._option_dict["interactor"]["class"]
         stimulator_kwargs = self._option_dict["interactor"]["kwargs"]
@@ -588,30 +606,11 @@ class ControlThread(Thread):
             else:
                 raise e
 
-        roi_builder = ROIBuilderClass(**roi_builder_kwargs)
-        
-        try:
-            reference_points, rois = roi_builder.build(cam)
-            
-            # Handle graceful failure when ROI building returns None values
-            if reference_points is None or rois is None:
-                logging.warning("ROI building failed: insufficient targets detected.")
-                # Save debug image to help user understand the issue
-                self._save_roi_debug_image(cam, "Insufficient targets detected")
-                try:
-                    cam._close()
-                    # Add a delay to allow camera hardware to reset
-                    time.sleep(2.0)
-                    logging.info("Camera cleanup completed, hardware should be available for next attempt")
-                except Exception as cleanup_error:
-                    logging.error(f"Error during camera cleanup: {cleanup_error}")
-                # Return None to indicate failure instead of raising exception
-                return None
-                
-        except (EthoscopeException, Exception) as e:
-            logging.error(f"ROI building failed: {e}")
-            # Save debug image with exception details
-            self._save_roi_debug_image(cam, f"ROI building error: {str(e)}")
+        # Use new shared target detection method
+        reference_points, rois = self._detect_and_store_targets(cam)
+
+        # Handle detection failure
+        if reference_points is None or rois is None:
             try:
                 cam._close()
                 # Add a delay to allow camera hardware to reset
@@ -864,6 +863,46 @@ class ControlThread(Thread):
         return  (cam, rw, rois, reference_points, TrackerClass, tracker_kwargs,
                         hardware_connection, StimulatorClass, stimulator_kwargs, time_offset)
 
+    def _detect_and_store_targets(self, cam):
+        """
+        Detect targets using ROI builder and store coordinates in experimental_info.
+
+        Args:
+            cam: Camera instance to use for detection
+
+        Returns:
+            tuple: (reference_points, rois) or (None, None) if detection failed
+        """
+        ROIBuilderClass = self._option_dict["roi_builder"]["class"]
+        roi_builder_kwargs = self._option_dict["roi_builder"]["kwargs"]
+
+        roi_builder = ROIBuilderClass(**roi_builder_kwargs)
+
+        try:
+            reference_points, rois = roi_builder.build(cam)
+
+            # Handle graceful failure when ROI building returns None values
+            if reference_points is None or rois is None:
+                logging.warning("ROI building failed: insufficient targets detected.")
+                # Save debug image to help user understand the issue
+                self._save_roi_debug_image(cam, "Insufficient targets detected")
+                return None, None
+
+            # Store target coordinates in experimental_info for API access
+            if "experimental_info" in self._info:
+                self._info["experimental_info"]["target_coordinates"] = [
+                    [float(p[0]), float(p[1])] for p in reference_points
+                ]
+                logging.info(f"Stored {len(reference_points)} target coordinates in experimental_info")
+
+            return reference_points, rois
+
+        except (EthoscopeException, Exception) as e:
+            logging.error(f"Target detection failed: {e}")
+            # Save debug image with exception details
+            self._save_roi_debug_image(cam, f"Target detection error: {str(e)}")
+            return None, None
+
     def _save_roi_debug_image(self, cam, error_message):
         """
         Save a debug image when ROI building fails to help user understand the issue.
@@ -871,19 +910,19 @@ class ControlThread(Thread):
         try:
             # Get a frame from the camera to show what was detected
             _, frame = next(iter(cam))
-            
+
             # Convert to color if it's grayscale for better annotation visibility
             if len(frame.shape) == 2:
                 debug_frame = cv2.cvtColor(frame, cv2.COLOR_GRAY2BGR)
             else:
                 debug_frame = frame.copy()
-            
+
             # Add timestamp in bottom right corner in white text
             font = cv2.FONT_HERSHEY_SIMPLEX
             font_scale = 2.0  # 4x larger than 0.5
             color = (255, 255, 255)  # White color
             thickness = 3  # Thicker for better visibility
-            
+
             # Get current timestamp with timezone
             timestamp = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S %Z")
             # If no timezone info, add local timezone indicator
@@ -891,26 +930,50 @@ class ControlThread(Thread):
                 import time
                 tz_name = time.tzname[time.daylight]
                 timestamp = f"{timestamp.rstrip()} {tz_name}"
-            
+
             # Calculate text size to position it in bottom right
             text_size = cv2.getTextSize(timestamp, font, font_scale, thickness)[0]
             text_x = debug_frame.shape[1] - text_size[0] - 10  # 10 pixels from right edge
             text_y = debug_frame.shape[0] - 10  # 10 pixels from bottom edge
-            
+
             cv2.putText(debug_frame, timestamp, (text_x, text_y), font, font_scale, color, thickness)
-            
+
             # Save the debug image
             debug_path = self._info["dbg_img"]
             cv2.imwrite(debug_path, debug_frame)
             logging.info(f"Debug image saved to: {debug_path}")
-            
+
         except Exception as e:
             logging.error(f"Failed to save debug image: {e}")
 
+    def _initialization_timeout_handler(self):
+        """
+        Emergency timeout handler that kills the ethoscope process if initialization takes too long.
+        This prevents the ethoscope from hanging indefinitely in 'initialising' state.
+        """
+        time.sleep(120)  # Wait 2 minutes
+        if self._info["status"] == "initialising":
+            logging.critical("INITIALIZATION TIMEOUT: Ethoscope has been stuck in initializing state for 2 minutes")
+            logging.critical("This usually indicates a camera hardware or picamera2 compatibility issue")
+            logging.critical("Terminating ethoscope process to prevent indefinite hang")
+            
+            # Set error state before terminating
+            self._info["status"] = "error"
+            self._info["error"] = "Initialization timeout: Process terminated after 2 minutes in initializing state"
+            self._info["time"] = time.time()
+            
+            # Force kill the current process
+            os.kill(os.getpid(), signal.SIGKILL)
+
     def run(self):
         cam = None
         hardware_connection = None
 
+        # Start timeout watchdog thread
+        timeout_thread = threading.Thread(target=self._initialization_timeout_handler, daemon=True)
+        timeout_thread.start()
+        logging.info("Started initialization timeout watchdog (2 minute limit)")
+
         try:
             self._info["status"] = "initialising"
             logging.info("Starting Monitor thread")
@@ -931,6 +994,9 @@ class ControlThread(Thread):
                 
             cam, rw, rois, reference_points, TrackerClass, tracker_kwargs, hardware_connection, StimulatorClass, stimulator_kwargs, time_offset = tracking_setup
             
+            # Initialization completed successfully
+            logging.info("Initialization completed successfully - starting tracking")
+            
             with rw as result_writer:
                 # Start tracking directly (pickle saving removed)
                 self._start_tracking(cam, result_writer, rois, reference_points, TrackerClass, tracker_kwargs, hardware_connection, StimulatorClass, stimulator_kwargs, time_offset=time_offset)
@@ -940,12 +1006,37 @@ class ControlThread(Thread):
         except EthoscopeException as e:
             if e.img is not  None:
                 cv2.imwrite(self._info["dbg_img"], e.img)
-            # This is an exception-based stop, so it's not graceful
-            self.stop(traceback.format_exc())
+            
+            # Check if this is a camera hardware issue that should not cause a permanent failure
+            error_msg = str(e).lower()
+            if ("camera hardware not available" in error_msg or 
+                "video tracking and recording are disabled" in error_msg or
+                "picamera2 compatibility" in error_msg or
+                "allocator" in error_msg):
+                logging.error(f"Camera initialization failed: {e}")
+                self._info["status"] = "stopped"
+                self._info["error"] = f"Camera initialization failed: {str(e)}"
+                self._info["time"] = time.time()
+                # Don't call stop() which would set error traceback - just clean exit
+                return
+            else:
+                # This is an exception-based stop, so it's not graceful
+                self.stop(traceback.format_exc())
         
         except Exception as e:
-            # This is an exception-based stop, so it's not graceful
-            self.stop(traceback.format_exc())
+            # Check if this is a camera-related exception that should not cause permanent failure
+            error_msg = str(e).lower()
+            if ("allocator" in error_msg or "picamera" in error_msg or 
+                "camera" in error_msg and "hardware" in error_msg):
+                logging.error(f"Camera-related error during initialization: {e}")
+                self._info["status"] = "stopped"
+                self._info["error"] = f"Camera error: {str(e)}"
+                self._info["time"] = time.time()
+                # Don't call stop() which would set error traceback - just clean exit
+                return
+            else:
+                # This is an exception-based stop, so it's not graceful
+                self.stop(traceback.format_exc())
 
         finally:
             try:
