@@ -828,42 +828,64 @@
                     }
                 }
 
-                // Replace the interactor section with MultiStimulator configuration
-                option.interactor = {
-                    name: 'MultiStimulator',
-                    arguments: {
-                        stimulator_sequence: $scope.stimulatorSequence.map(function(stim) {
-                            // Create a clean copy of arguments without the date_range
-                            var cleanArguments = {};
-                            if (stim.arguments) {
-                                for (var key in stim.arguments) {
-                                    if (key !== 'date_range') {
-                                        cleanArguments[key] = stim.arguments[key];
+                // If only one stimulator, use it directly without MultiStimulator wrapper
+                if ($scope.stimulatorSequence.length === 1) {
+                    var singleStimulator = $scope.stimulatorSequence[0];
+                    
+                    // Create arguments object including date_range
+                    var stimulatorArguments = {};
+                    if (singleStimulator.arguments) {
+                        for (var key in singleStimulator.arguments) {
+                            stimulatorArguments[key] = singleStimulator.arguments[key];
+                        }
+                    }
+                    
+                    option.interactor = {
+                        name: singleStimulator.name,
+                        arguments: stimulatorArguments
+                    };
+                    
+                    console.log('Configured single stimulator:', singleStimulator.name, 'with arguments:', stimulatorArguments);
+                    
+                } else {
+                    // Multiple stimulators - use MultiStimulator configuration
+                    option.interactor = {
+                        name: 'MultiStimulator',
+                        arguments: {
+                            stimulator_sequence: $scope.stimulatorSequence.map(function(stim) {
+                                // Create a clean copy of arguments without the date_range
+                                var cleanArguments = {};
+                                if (stim.arguments) {
+                                    for (var key in stim.arguments) {
+                                        if (key !== 'date_range') {
+                                            cleanArguments[key] = stim.arguments[key];
+                                        }
                                     }
                                 }
-                            }
 
-                            return {
-                                class_name: stim.name,
-                                arguments: cleanArguments,
-                                date_range: (stim.arguments && stim.arguments.date_range) ?
-                                    (typeof stim.arguments.date_range === 'string' ? stim.arguments.date_range : '') : ''
-                            };
-                        })
-                    }
-                };
-                // Sanitize the data to prevent JSON errors
-                try {
-                    JSON.stringify(option.interactor.arguments.stimulator_sequence);
-                    console.log('Configured MultiStimulator with sequence:', option.interactor.arguments.stimulator_sequence);
-                } catch (jsonError) {
-                    console.error('JSON serialization error in stimulator sequence:', jsonError);
-                    console.log('Raw stimulator sequence:', $scope.stimulatorSequence);
-                    // Fallback to DefaultStimulator if JSON serialization fails
-                    option.interactor = {
-                        name: 'DefaultStimulator',
-                        arguments: {}
+                                return {
+                                    class_name: stim.name,
+                                    arguments: cleanArguments,
+                                    date_range: (stim.arguments && stim.arguments.date_range) ?
+                                        (typeof stim.arguments.date_range === 'string' ? stim.arguments.date_range : '') : ''
+                                };
+                            })
+                        }
                     };
+                    
+                    // Sanitize the data to prevent JSON errors
+                    try {
+                        JSON.stringify(option.interactor.arguments.stimulator_sequence);
+                        console.log('Configured MultiStimulator with sequence:', option.interactor.arguments.stimulator_sequence);
+                    } catch (jsonError) {
+                        console.error('JSON serialization error in stimulator sequence:', jsonError);
+                        console.log('Raw stimulator sequence:', $scope.stimulatorSequence);
+                        // Fallback to DefaultStimulator if JSON serialization fails
+                        option.interactor = {
+                            name: 'DefaultStimulator',
+                            arguments: {}
+                        };
+                    }
                 }
             } else {
                 // If no stimulators in sequence, use DefaultStimulator
@@ -1748,6 +1770,149 @@
             });
         }, 100);
 
+        // ===========================
+        // STIMULATOR STATUS FUNCTIONS
+        // ===========================
+
+        /**
+         * Get a human-readable stimulator name from the class name
+         */
+        $scope.getReadableStimulatorName = function(className) {
+            if (!className) return 'None';
+            
+            // Extract class name from full path format
+            var match = className.match(/class '([^']+)'/);
+            if (match) {
+                className = match[1];
+            }
+            
+            // Extract just the class name without module path
+            var parts = className.split('.');
+            var shortName = parts[parts.length - 1];
+            
+            // Convert common stimulator names to readable format
+            var nameMapping = {
+                'DefaultStimulator': 'Default (No Stimulation)',
+                'MultiStimulator': 'Multi-Stimulator Sequence',
+                'mAGO': 'mAGO Sleep Depriver',
+                'AGO': 'AGO Sleep Depriver',
+                'SleepDepStimulator': 'Sleep Deprivation',
+                'OptomotorSleepDepriver': 'Optomotor Sleep Depriver',
+                'MiddleCrossingStimulator': 'Middle Crossing Stimulator',
+                'ExperimentalSleepDepStimulator': 'Experimental Sleep Depriver',
+                'DynamicOdourSleepDepriver': 'Dynamic Odour Sleep Depriver',
+                'OptoMidlineCrossStimulator': 'Optomotor Midline Cross',
+                'OptomotorSleepDepriverSystematic': 'Systematic Optomotor Sleep Depriver',
+                'MiddleCrossingOdourStimulator': 'Middle Crossing Odour Stimulator',
+                'MiddleCrossingOdourStimulatorFlushed': 'Flushed Odour Stimulator'
+            };
+            
+            return nameMapping[shortName] || shortName;
+        };
+
+        /**
+         * Format date range string for display
+         */
+        $scope.formatDateRange = function(dateRange) {
+            if (!dateRange || dateRange.trim() === '') {
+                return 'Always Active';
+            }
+            
+            try {
+                // Parse the date range format "YYYY-MM-DD HH:mm:ss > YYYY-MM-DD HH:mm:ss"
+                var parts = dateRange.split('>');
+                if (parts.length === 2) {
+                    var startStr = parts[0].trim();
+                    var endStr = parts[1].trim();
+                    
+                    if (startStr && endStr) {
+                        // Format dates to be more readable
+                        var startDate = new Date(startStr.replace(/ /, 'T'));
+                        var endDate = new Date(endStr.replace(/ /, 'T'));
+                        
+                        var formatDate = function(date) {
+                            return date.toLocaleDateString() + ' ' + date.toLocaleTimeString([], {hour: '2-digit', minute:'2-digit'});
+                        };
+                        
+                        return formatDate(startDate) + ' → ' + formatDate(endDate);
+                    }
+                }
+                
+                return dateRange;
+            } catch (e) {
+                return dateRange;
+            }
+        };
+
+        /**
+         * Check if stimulator is currently scheduled to be active
+         */
+        $scope.isStimulatorScheduled = function() {
+            if (!$scope.device || !$scope.device.experimental_info || 
+                !$scope.device.experimental_info.current || 
+                !$scope.device.experimental_info.current.interactor) {
+                return false;
+            }
+            
+            var dateRange = $scope.device.experimental_info.current.interactor.arguments.date_range;
+            if (!dateRange || dateRange.trim() === '') {
+                return true; // Always active if no date range specified
+            }
+            
+            try {
+                var parts = dateRange.split('>');
+                if (parts.length === 2) {
+                    var startStr = parts[0].trim();
+                    var endStr = parts[1].trim();
+                    
+                    if (startStr && endStr) {
+                        var startDate = new Date(startStr.replace(/ /, 'T'));
+                        var endDate = new Date(endStr.replace(/ /, 'T'));
+                        var now = new Date();
+                        
+                        return now >= startDate && now <= endDate;
+                    }
+                }
+            } catch (e) {
+                console.error('Error parsing date range:', e);
+            }
+            
+            return false;
+        };
+
+        /**
+         * Get device time as a formatted string
+         */
+        $scope.getDeviceTimeString = function() {
+            if ($scope.device_datetime) {
+                return $scope.device_datetime;
+            }
+            return new Date().toLocaleString();
+        };
+
+        /**
+         * Format configuration values for display
+         */
+        $scope.formatConfigValue = function(value) {
+            if (value === null || value === undefined) {
+                return 'null';
+            }
+            
+            if (typeof value === 'object') {
+                try {
+                    return JSON.stringify(value, null, 2);
+                } catch (e) {
+                    return '[Object]';
+                }
+            }
+            
+            if (typeof value === 'string' && value.length > 100) {
+                return value.substring(0, 100) + '...';
+            }
+            
+            return String(value);
+        };
+
     });
 
 })();
\ No newline at end of file
